import { EventEmitter, Injectable, Directive, ElementRef, HostListener, Input, Optional, Output, Renderer2, TemplateRef, ViewContainerRef, NgModule } from '@angular/core';
import { FormGroup, FormsModule, ReactiveFormsModule, FormControl } from '@angular/forms';
import { takeWhile } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

class NrfNestedFormService {
    constructor() {
        this.submit$ = new EventEmitter();
        this.formGroup = new FormGroup({});
    }
    /**
     * @param {?} entity
     * @return {?}
     */
    set entity(entity) {
        this.entityInternal = entity;
        this.formData = this.cloneDeep(entity); // TODO merge with existent formData
    }
    /**
     * @return {?}
     */
    get entity() {
        return this.entityInternal;
    }
    /**
     * @param {?} target
     * @return {?}
     */
    cloneDeep(target) {
        if (!target || typeof target !== 'object') {
            return target;
        }
        if (target instanceof Date) {
            return new Date(target);
        }
        if (Array.isArray(target)) {
            return target.map((value) => this.cloneDeep(value));
        }
        return Object.keys(target).reduce((props, key) => {
            props[key] = this.cloneDeep(target[key]);
            return props;
        }, {});
    }
}
NrfNestedFormService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NrfNestedFormService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * A component to abstract the form implementation
 *
 * It holds the validation and the submit logic
 *
 * \@example
 * <form nrfForm (nrfSubmit)="aCallback($event)">
 *  // Inputs
 * </form>
 */
class NrfFormDirective {
    /**
     * @ignore
     * @param {?} nestedFormService
     * @param {?} templateRef
     * @param {?} viewContainerRef
     * @param {?} el
     * @param {?} renderer
     */
    constructor(nestedFormService, templateRef, viewContainerRef, el, renderer) {
        this.nestedFormService = nestedFormService;
        this.templateRef = templateRef;
        this.viewContainerRef = viewContainerRef;
        this.el = el;
        this.renderer = renderer;
        if (!this.nestedFormService) {
            this.nestedFormService = new NrfNestedFormService();
        }
    }
    /**
     * Represents an Entity Model that comes from Database
     * @param {?} entity
     * @return {?}
     */
    set nrfEntity(entity) {
        if (this.nestedFormService.entity !== entity) {
            this.nestedFormService.entity = entity;
        }
    }
    /**
     * @return {?}
     */
    get nrfEntity() {
        return this.nestedFormService.entity;
    }
    /**
     * @return {?}
     */
    get formData() {
        return this.nestedFormService.formData;
    }
    /**
     * A function that will be called when the form is valid and a submit event is triggered
     * by a button click or programmatically.
     * @return {?}
     */
    get nrfSubmit() {
        return this.nestedFormService.submit$;
    }
    /**
     * Form group will hold all inputs within this form.
     *
     * Every input should register itself to this form group.
     *
     * The data in this form group will not be sent to backend, just form validations and input management.
     * @return {?}
     */
    get formGroup() {
        return this.nestedFormService.formGroup;
    }
    /**
     * Init an empty nrfEntity, in case none where provided.
     * @return {?}
     */
    ngOnInit() {
        if (!this.nrfEntity) {
            this.nrfEntity = {};
        }
        this.renderView();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.nrfSubmit.complete();
    }
    /**
     * @return {?}
     */
    renderView() {
        if (this.templateRef && this.viewContainerRef) {
            const /** @type {?} */ embeddedViewRef = this.viewContainerRef.createEmbeddedView(this.templateRef);
            const /** @type {?} */ formNative = embeddedViewRef.rootNodes[0];
            this.renderer.listen(formNative, 'submit', (event) => this.formSubmitWrapper(event));
        }
    }
    /**
     * # Should not be called directly!
     * This method wraps the form validation and call [nrfSubmit]{\@link NrfFormDirective#nrfSubmit}
     * @param {?} $event
     * @return {?}
     */
    formSubmitWrapper($event) {
        $event.preventDefault();
        if (!this.formGroup.valid) {
            return;
        }
        this.nrfSubmit.emit({
            nrfForm: this,
            formData: this.formData,
            entity: this.nrfEntity,
            formGroup: this.formGroup,
            event: $event,
        });
    }
}
NrfFormDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nrfForm]',
                exportAs: 'nrfForm',
            },] },
];
/** @nocollapse */
NrfFormDirective.ctorParameters = () => [
    { type: NrfNestedFormService, decorators: [{ type: Optional },] },
    { type: TemplateRef, decorators: [{ type: Optional },] },
    { type: ViewContainerRef, decorators: [{ type: Optional },] },
    { type: ElementRef, },
    { type: Renderer2, },
];
NrfFormDirective.propDecorators = {
    "nrfEntity": [{ type: Input },],
    "nrfSubmit": [{ type: Output },],
    "formSubmitWrapper": [{ type: HostListener, args: ['submit', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * This module only handle form Abstraction and submit handler
 * No inputs are provided by this module.
 */
class NrfFormModule {
}
NrfFormModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    FormsModule,
                    ReactiveFormsModule,
                ],
                exports: [
                    FormsModule,
                    ReactiveFormsModule,
                    NrfFormDirective,
                ],
                declarations: [
                    NrfFormDirective,
                ],
                providers: [
                    NrfNestedFormService,
                ],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Class to abstract get and set values from an object or an array
 * using dot-notation string as the path to the property
 *
 */
class NrfModelSetterService {
    constructor() {
        /**
         * Get the final property of the dot notation path.
         * This will be the property that will receive the new value
         */
        this.getTargetPropToSet = (obj, key, i, pathPieces) => {
            let /** @type {?} */ nextKey = pathPieces[i + 1];
            const /** @type {?} */ isLast = nextKey == null;
            nextKey = nextKey || pathPieces['finalKey'];
            const /** @type {?} */ isArrayKey = this.isArrayKey(nextKey);
            let /** @type {?} */ prop = obj[key];
            if (!prop) {
                prop = isArrayKey ? [] : {};
                obj[key] = prop;
            }
            else if (isLast) {
                prop = isArrayKey ? [...prop] : Object.assign({}, prop);
                obj[key] = prop;
            }
            return prop;
        };
    }
    /**
     * Get a value using a dot-notation string to find the nested property
     *
     * @param {?} path Dot notation of the value to be get
     * @param {?} model An object or an Array
     * @param {?=} separator An optional param to specify a different separator, default '.'
     *
     * ```typescript
     * const nrfEntity = {
     *    user: {
     *        name: 'John Doe'
     *    }
     * };
     * nestedProps.getValue('user.name', nrfEntity);
     * // => 'John Doe'
     *
     * nestedProps.getValue('user.email', nrfEntity);
     * // => null
     * ```
     * @return {?}
     */
    getValue(path, model, separator = '.') {
        if (path == null || !model) {
            return null;
        }
        try {
            const /** @type {?} */ pathPieces = this.generatePathPieces(path, separator);
            return pathPieces.reduce(this.piecesReducer, model) || null;
        }
        catch (/** @type {?} */ err) {
            return null;
        }
    }
    /**
     * @ignore
     *
     * Return the value associated to the given key.
     *
     * If the key is empty, the last element of an array is returned
     * @param {?} obj
     * @param {?} key
     * @return {?}
     */
    piecesReducer(obj, key) {
        return key === '' ? obj.slice(-1)[0] : obj[key];
    }
    /**
     * Set a value using a dot-notation string.
     *
     * If the property does not exists, the entire path will be created.
     *
     * @param {?} path Dot notation of the value to be get
     * @param {?} value The value to be set
     * @param {?} model An object or an Array
     * @param {?=} separator An optional param to specify a different separator, default '.'
     *
     * ```typescript
     * nestedProps.setValue('user.age', 35, { user: { name: 'John Doe' } });
     * ```
     * @return {?}
     */
    setValue(path, value, model, separator = '.') {
        if (!(path && model)) {
            return null;
        }
        try {
            const /** @type {?} */ pathPieces = this.generatePathPieces(path, separator);
            const /** @type {?} */ finalKey = pathPieces.pop();
            (/** @type {?} */ (pathPieces))['finalKey'] = finalKey;
            const /** @type {?} */ targetProp = pathPieces.reduce(this.getTargetPropToSet, model) || null;
            if (finalKey === '') {
                targetProp.push(value);
            }
            else {
                targetProp[finalKey] = value;
            }
        }
        catch (/** @type {?} */ err) {
            console.error('NestedProps setValue error: ', err);
        }
    }
    /**
     * @ignore
     * Internal function to define if a given key is for an array or an object
     * @param {?} key
     * @return {?}
     */
    isArrayKey(key) {
        return key === '' || !isNaN(parseInt(key, 10));
    }
    /**
     * Generate the dot-notation path as an Array to iterate over.
     * @param {?} path
     * @param {?} separator
     * @return {?}
     */
    generatePathPieces(path, separator) {
        return path
            .replace('[', separator)
            .replace(']', '')
            .split(separator);
    }
}
NrfModelSetterService.decorators = [
    { type: Injectable },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NrfNestedControlContext {
    /**
     * @param {?} formControl
     * @param {?} formGroup
     * @param {?} nrfNestedControl
     */
    constructor(formControl, formGroup, nrfNestedControl) {
        this.formControl = formControl;
        this.formGroup = formGroup;
        this.nrfNestedControl = nrfNestedControl;
        this.$implicit = formControl;
    }
}
/**
 * This directive control nested inputs and sets values on the Original Model set at {\@link NrfFormDirective#nrfEntity}
 *
 * #### Given an nrfEntity on the controller:
 * ```typescript
 * this.userModel = {
 *    firstName: 'John',
 *    address: {
 *        street: 'Carnaby Street'
 *    }
 * };
 * ```
 *
 * #### Use it on the form
 * ```html
 * <form [nrfEntity]="userModel">
 *   <div class="form-group">
 *      <label for="name">Name:</label>
 *
 *      <div *nrfNestedControl="'userModel.firstName'; let control=formControl">
 *        <input [formControl]="control" />
 *      </div>
 *   </div>
 * </form>
 * ```
 */
class NrfNestedControlDirective {
    /**
     * @param {?} modelSetter
     * @param {?} nestedFormService
     * @param {?} templateRef
     * @param {?} viewContainerRef
     */
    constructor(modelSetter, nestedFormService, templateRef, viewContainerRef) {
        this.modelSetter = modelSetter;
        this.nestedFormService = nestedFormService;
        this.templateRef = templateRef;
        this.viewContainerRef = viewContainerRef;
        this.ready$ = new EventEmitter();
        this.isRegisteredToFormControl = false;
        this.isDestroyed = false;
    }
    /**
     * Register this input component to its parent form [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
     *
     * And starts to emit the input's value when it changes.
     * @return {?}
     */
    ngOnInit() {
        this.modelPath = this.getModelPathWithoutFirstPart();
        this.registerToFormGroup();
        this.subscribeToValueChanges();
        this.showViewContent();
        this.emitReadyState();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.isDestroyed = true;
        this.ready$.complete();
        if (this.parentFormGroup) {
            this.parentFormGroup.removeControl(this.nrfModelName);
        }
    }
    /**
     * @return {?}
     */
    showViewContent() {
        const /** @type {?} */ context = new NrfNestedControlContext(this.formControl, this.parentFormGroup, this);
        this.viewContainerRef.createEmbeddedView(this.templateRef, context);
    }
    /**
     * Register this input to its parent [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
     * to enable validations and data manipulation
     * @return {?}
     */
    registerToFormGroup() {
        if (this.isRegisteredToFormControl) {
            return;
        }
        const /** @type {?} */ formGroup = this.getFormGroup();
        const /** @type {?} */ formControl = this.getFormControl(formGroup);
        this.formControl = formControl;
        if (formGroup) {
            this.isRegisteredToFormControl = true;
            this.parentFormGroup = formGroup;
            formGroup.addControl(this.nrfModelName, formControl);
        }
    }
    /**
     * Verify if this input is inside a [NrfFormDirective]{\@link NrfFormDirective}
     * and return its [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
     *
     * Otherwise a new empty [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
     * @return {?}
     */
    getFormGroup() {
        if (this.nestedFormService) {
            return this.nestedFormService.formGroup;
        }
        return new FormGroup({});
    }
    /**
     * Verify if the [FormGroup]{\@link https://angular.io/api/forms/FormGroup} has an control with the current name and return it.
     * Otherwise return a new [FormControl]{\@link https://angular.io/api/forms/FormControl}
     * @param {?} formGroup
     * @return {?}
     */
    getFormControl(formGroup) {
        let /** @type {?} */ formControl = formGroup && /** @type {?} */ (formGroup.get(this.nrfModelName));
        if (!formControl) {
            formControl = this.getNewFormControl();
        }
        return formControl;
    }
    /**
     * Instantiate a new [FormControl]{\@link https://angular.io/api/forms/FormControl} and return it
     * @return {?}
     */
    getNewFormControl() {
        const /** @type {?} */ initialValue = this.getInitialValue();
        return new FormControl(initialValue || null);
    }
    /**
     * Subscribe to [valueChanges]{\@link https://angular.io/api/forms/AbstractControl#valueChanges} and update the Entity value
     * @return {?}
     */
    subscribeToValueChanges() {
        this.formControl.valueChanges
            .pipe(takeWhile(() => !this.isDestroyed))
            .subscribe(newValue => this.setModelValue(newValue));
    }
    /**
     * Return the dot notation path of the Entity, without the first part,
     * because it is the Entity itself.
     * @return {?}
     */
    getModelPathWithoutFirstPart() {
        return this.nrfModelName.substr(this.nrfModelName.indexOf('.') + 1);
    }
    /**
     * Get the value from the [nrfEntity]{\@link NrfFormDirective#nrfEntity}
     * @return {?}
     */
    getInitialValue() {
        if (this.nestedFormService) {
            return this.modelSetter.getValue(this.modelPath, this.nestedFormService.formData);
        }
        return null;
    }
    /**
     * Set the value to the [formData]{\@link NrfFormDirective#formData}
     * @param {?} newValue
     * @return {?}
     */
    setModelValue(newValue) {
        if (this.nestedFormService) {
            return this.modelSetter.setValue(this.modelPath, newValue || '', this.nestedFormService.formData);
        }
    }
    /**
     * @return {?}
     */
    emitReadyState() {
        this.ready$.emit(true);
    }
}
NrfNestedControlDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nrfNestedControl]',
                exportAs: 'nrfNestedControl',
            },] },
];
/** @nocollapse */
NrfNestedControlDirective.ctorParameters = () => [
    { type: NrfModelSetterService, },
    { type: NrfNestedFormService, decorators: [{ type: Optional },] },
    { type: TemplateRef, },
    { type: ViewContainerRef, },
];
NrfNestedControlDirective.propDecorators = {
    "nrfModelName": [{ type: Input, args: ['nrfNestedControl',] },],
    "ready$": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Holds the nrfModel directive implementation
 */
class NrfModelModule {
}
NrfModelModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                providers: [
                    NrfModelSetterService,
                    NrfNestedFormService,
                ],
                declarations: [
                    NrfNestedControlDirective,
                ],
                exports: [
                    NrfNestedControlDirective,
                ],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NrFormsModule {
}
NrFormsModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    NrfFormModule,
                    NrfModelModule,
                ],
                providers: [
                    NrfNestedFormService,
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/*
 * Public API Surface of ng-nrforms
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { NrFormsModule, NrfFormDirective, NrfNestedControlContext, NrfNestedControlDirective, NrfModelSetterService, NrfNestedFormService, NrfFormModule as ɵa, NrfModelModule as ɵb };
//# sourceMappingURL=webdev-tools-ng-nested-reactive-forms.js.map
