import { __spread } from 'tslib';
import { EventEmitter, Injectable, Directive, ElementRef, HostListener, Input, Optional, Output, Renderer2, TemplateRef, ViewContainerRef, NgModule } from '@angular/core';
import { FormGroup, FormsModule, ReactiveFormsModule, FormControl } from '@angular/forms';
import { takeWhile } from 'rxjs/operators';

var NrfNestedFormService = /** @class */ (function () {
    function NrfNestedFormService() {
        this.submit$ = new EventEmitter();
        this.formGroup = new FormGroup({});
    }
    Object.defineProperty(NrfNestedFormService.prototype, "entity", {
        get: function () {
            return this.entityInternal;
        },
        set: function (entity) {
            this.entityInternal = entity;
            this.formData = this.cloneDeep(entity);
        },
        enumerable: true,
        configurable: true
    });
    NrfNestedFormService.prototype.cloneDeep = function (target) {
        var _this = this;
        if (!target || typeof target !== 'object') {
            return target;
        }
        if (target instanceof Date) {
            return new Date(target);
        }
        if (Array.isArray(target)) {
            return target.map(function (value) { return _this.cloneDeep(value); });
        }
        return Object.keys(target).reduce(function (props, key) {
            props[key] = _this.cloneDeep(target[key]);
            return props;
        }, {});
    };
    return NrfNestedFormService;
}());
NrfNestedFormService.decorators = [
    { type: Injectable },
];
NrfNestedFormService.ctorParameters = function () { return []; };
var NrfFormDirective = /** @class */ (function () {
    function NrfFormDirective(nestedFormService, templateRef, viewContainerRef, el, renderer) {
        this.nestedFormService = nestedFormService;
        this.templateRef = templateRef;
        this.viewContainerRef = viewContainerRef;
        this.el = el;
        this.renderer = renderer;
        if (!this.nestedFormService) {
            this.nestedFormService = new NrfNestedFormService();
        }
    }
    Object.defineProperty(NrfFormDirective.prototype, "nrfEntity", {
        get: function () {
            return this.nestedFormService.entity;
        },
        set: function (entity) {
            if (this.nestedFormService.entity !== entity) {
                this.nestedFormService.entity = entity;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NrfFormDirective.prototype, "formData", {
        get: function () {
            return this.nestedFormService.formData;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NrfFormDirective.prototype, "nrfSubmit", {
        get: function () {
            return this.nestedFormService.submit$;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NrfFormDirective.prototype, "formGroup", {
        get: function () {
            return this.nestedFormService.formGroup;
        },
        enumerable: true,
        configurable: true
    });
    NrfFormDirective.prototype.ngOnInit = function () {
        if (!this.nrfEntity) {
            this.nrfEntity = {};
        }
        this.renderView();
    };
    NrfFormDirective.prototype.ngOnDestroy = function () {
        this.nrfSubmit.complete();
    };
    NrfFormDirective.prototype.renderView = function () {
        var _this = this;
        if (this.templateRef && this.viewContainerRef) {
            var embeddedViewRef = this.viewContainerRef.createEmbeddedView(this.templateRef);
            var formNative = embeddedViewRef.rootNodes[0];
            this.renderer.listen(formNative, 'submit', function (event) { return _this.formSubmitWrapper(event); });
        }
    };
    NrfFormDirective.prototype.formSubmitWrapper = function ($event) {
        $event.preventDefault();
        if (!this.formGroup.valid) {
            return;
        }
        this.nrfSubmit.emit({
            nrfForm: this,
            formData: this.formData,
            entity: this.nrfEntity,
            formGroup: this.formGroup,
            event: $event,
        });
    };
    return NrfFormDirective;
}());
NrfFormDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nrfForm]',
                exportAs: 'nrfForm',
            },] },
];
NrfFormDirective.ctorParameters = function () { return [
    { type: NrfNestedFormService, decorators: [{ type: Optional },] },
    { type: TemplateRef, decorators: [{ type: Optional },] },
    { type: ViewContainerRef, decorators: [{ type: Optional },] },
    { type: ElementRef, },
    { type: Renderer2, },
]; };
NrfFormDirective.propDecorators = {
    "nrfEntity": [{ type: Input },],
    "nrfSubmit": [{ type: Output },],
    "formSubmitWrapper": [{ type: HostListener, args: ['submit', ['$event'],] },],
};
var NrfFormModule = /** @class */ (function () {
    function NrfFormModule() {
    }
    return NrfFormModule;
}());
NrfFormModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    FormsModule,
                    ReactiveFormsModule,
                ],
                exports: [
                    FormsModule,
                    ReactiveFormsModule,
                    NrfFormDirective,
                ],
                declarations: [
                    NrfFormDirective,
                ],
                providers: [
                    NrfNestedFormService,
                ],
            },] },
];
var NrfModelSetterService = /** @class */ (function () {
    function NrfModelSetterService() {
        var _this = this;
        this.getTargetPropToSet = function (obj, key, i, pathPieces) {
            var nextKey = pathPieces[i + 1];
            var isLast = nextKey == null;
            nextKey = nextKey || pathPieces['finalKey'];
            var isArrayKey = _this.isArrayKey(nextKey);
            var prop = obj[key];
            if (!prop) {
                prop = isArrayKey ? [] : {};
                obj[key] = prop;
            }
            else if (isLast) {
                prop = isArrayKey ? __spread(prop) : Object.assign({}, prop);
                obj[key] = prop;
            }
            return prop;
        };
    }
    NrfModelSetterService.prototype.getValue = function (path, model, separator) {
        if (separator === void 0) { separator = '.'; }
        if (path == null || !model) {
            return null;
        }
        try {
            var pathPieces = this.generatePathPieces(path, separator);
            return pathPieces.reduce(this.piecesReducer, model) || null;
        }
        catch (err) {
            return null;
        }
    };
    NrfModelSetterService.prototype.piecesReducer = function (obj, key) {
        return key === '' ? obj.slice(-1)[0] : obj[key];
    };
    NrfModelSetterService.prototype.setValue = function (path, value, model, separator) {
        if (separator === void 0) { separator = '.'; }
        if (!(path && model)) {
            return null;
        }
        try {
            var pathPieces = this.generatePathPieces(path, separator);
            var finalKey = pathPieces.pop();
            ((pathPieces))['finalKey'] = finalKey;
            var targetProp = pathPieces.reduce(this.getTargetPropToSet, model) || null;
            if (finalKey === '') {
                targetProp.push(value);
            }
            else {
                targetProp[finalKey] = value;
            }
        }
        catch (err) {
            console.error('NestedProps setValue error: ', err);
        }
    };
    NrfModelSetterService.prototype.isArrayKey = function (key) {
        return key === '' || !isNaN(parseInt(key, 10));
    };
    NrfModelSetterService.prototype.generatePathPieces = function (path, separator) {
        return path
            .replace('[', separator)
            .replace(']', '')
            .split(separator);
    };
    return NrfModelSetterService;
}());
NrfModelSetterService.decorators = [
    { type: Injectable },
];
var NrfNestedControlContext = /** @class */ (function () {
    function NrfNestedControlContext(formControl, formGroup, nrfNestedControl) {
        this.formControl = formControl;
        this.formGroup = formGroup;
        this.nrfNestedControl = nrfNestedControl;
        this.$implicit = formControl;
    }
    return NrfNestedControlContext;
}());
var NrfNestedControlDirective = /** @class */ (function () {
    function NrfNestedControlDirective(modelSetter, nestedFormService, templateRef, viewContainerRef) {
        this.modelSetter = modelSetter;
        this.nestedFormService = nestedFormService;
        this.templateRef = templateRef;
        this.viewContainerRef = viewContainerRef;
        this.ready$ = new EventEmitter();
        this.isRegisteredToFormControl = false;
        this.isDestroyed = false;
    }
    NrfNestedControlDirective.prototype.ngOnInit = function () {
        this.modelPath = this.getModelPathWithoutFirstPart();
        this.registerToFormGroup();
        this.subscribeToValueChanges();
        this.showViewContent();
        this.emitReadyState();
    };
    NrfNestedControlDirective.prototype.ngOnDestroy = function () {
        this.isDestroyed = true;
        this.ready$.complete();
        if (this.parentFormGroup) {
            this.parentFormGroup.removeControl(this.nrfModelName);
        }
    };
    NrfNestedControlDirective.prototype.showViewContent = function () {
        var context = new NrfNestedControlContext(this.formControl, this.parentFormGroup, this);
        this.viewContainerRef.createEmbeddedView(this.templateRef, context);
    };
    NrfNestedControlDirective.prototype.registerToFormGroup = function () {
        if (this.isRegisteredToFormControl) {
            return;
        }
        var formGroup = this.getFormGroup();
        var formControl = this.getFormControl(formGroup);
        this.formControl = formControl;
        if (formGroup) {
            this.isRegisteredToFormControl = true;
            this.parentFormGroup = formGroup;
            formGroup.addControl(this.nrfModelName, formControl);
        }
    };
    NrfNestedControlDirective.prototype.getFormGroup = function () {
        if (this.nestedFormService) {
            return this.nestedFormService.formGroup;
        }
        return new FormGroup({});
    };
    NrfNestedControlDirective.prototype.getFormControl = function (formGroup) {
        var formControl = formGroup && (formGroup.get(this.nrfModelName));
        if (!formControl) {
            formControl = this.getNewFormControl();
        }
        return formControl;
    };
    NrfNestedControlDirective.prototype.getNewFormControl = function () {
        var initialValue = this.getInitialValue();
        return new FormControl(initialValue || null);
    };
    NrfNestedControlDirective.prototype.subscribeToValueChanges = function () {
        var _this = this;
        this.formControl.valueChanges
            .pipe(takeWhile(function () { return !_this.isDestroyed; }))
            .subscribe(function (newValue) { return _this.setModelValue(newValue); });
    };
    NrfNestedControlDirective.prototype.getModelPathWithoutFirstPart = function () {
        return this.nrfModelName.substr(this.nrfModelName.indexOf('.') + 1);
    };
    NrfNestedControlDirective.prototype.getInitialValue = function () {
        if (this.nestedFormService) {
            return this.modelSetter.getValue(this.modelPath, this.nestedFormService.formData);
        }
        return null;
    };
    NrfNestedControlDirective.prototype.setModelValue = function (newValue) {
        if (this.nestedFormService) {
            return this.modelSetter.setValue(this.modelPath, newValue || '', this.nestedFormService.formData);
        }
    };
    NrfNestedControlDirective.prototype.emitReadyState = function () {
        this.ready$.emit(true);
    };
    return NrfNestedControlDirective;
}());
NrfNestedControlDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nrfNestedControl]',
                exportAs: 'nrfNestedControl',
            },] },
];
NrfNestedControlDirective.ctorParameters = function () { return [
    { type: NrfModelSetterService, },
    { type: NrfNestedFormService, decorators: [{ type: Optional },] },
    { type: TemplateRef, },
    { type: ViewContainerRef, },
]; };
NrfNestedControlDirective.propDecorators = {
    "nrfModelName": [{ type: Input, args: ['nrfNestedControl',] },],
    "ready$": [{ type: Output },],
};
var NrfModelModule = /** @class */ (function () {
    function NrfModelModule() {
    }
    return NrfModelModule;
}());
NrfModelModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                providers: [
                    NrfModelSetterService,
                    NrfNestedFormService,
                ],
                declarations: [
                    NrfNestedControlDirective,
                ],
                exports: [
                    NrfNestedControlDirective,
                ],
            },] },
];
var NrFormsModule = /** @class */ (function () {
    function NrFormsModule() {
    }
    return NrFormsModule;
}());
NrFormsModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    NrfFormModule,
                    NrfModelModule,
                ],
                providers: [
                    NrfNestedFormService,
                ]
            },] },
];

export { NrFormsModule, NrfFormDirective, NrfNestedControlContext, NrfNestedControlDirective, NrfModelSetterService, NrfNestedFormService, NrfFormModule as ɵa, NrfModelModule as ɵb };
//# sourceMappingURL=webdev-tools-ng-nested-reactive-forms.js.map
