import { ElementRef, EventEmitter, OnDestroy, OnInit, Renderer2, TemplateRef, ViewContainerRef } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { NrfNestedFormService, NrfSubmitData } from './nested-form.service';
/**
 * A component to abstract the form implementation
 *
 * It holds the validation and the submit logic
 *
 * @example
 * <form nrfForm (nrfSubmit)="aCallback($event)">
 *  // Inputs
 * </form>
 */
export declare class NrfFormDirective implements OnInit, OnDestroy {
    private readonly nestedFormService;
    private templateRef;
    private viewContainerRef;
    private el;
    private renderer;
    /**
     * Represents an Entity Model that comes from Database
     */
    nrfEntity: any;
    readonly formData: any;
    /**
     * A function that will be called when the form is valid and a submit event is triggered
     * by a button click or programmatically.
     */
    readonly nrfSubmit: EventEmitter<NrfSubmitData>;
    /**
     * Form group will hold all inputs within this form.
     *
     * Every input should register itself to this form group.
     *
     * The data in this form group will not be sent to backend, just form validations and input management.
     */
    readonly formGroup: FormGroup;
    /**
     * @ignore
     */
    constructor(nestedFormService: NrfNestedFormService, templateRef: TemplateRef<any>, viewContainerRef: ViewContainerRef, el: ElementRef, renderer: Renderer2);
    /**
     * Init an empty nrfEntity, in case none where provided.
     */
    ngOnInit(): void;
    ngOnDestroy(): void;
    private renderView();
    /**
     * # Should not be called directly!
     * This method wraps the form validation and call [nrfSubmit]{@link NrfFormDirective#nrfSubmit}
     */
    formSubmitWrapper($event: Event): void;
}
